from instagramy import InstagramUser
from instagramy import InstagramPost
from instagramy import InstagramHashTag


def test_user():
    user = InstagramUser('github')
    assert user.username == "github"

def test_tag():
    tag = InstagramHashTag('dog')
    assert tag.tagname == 'dog'

def test_post():
    post = InstagramPost('CGeYX2OA61s')
    assert post.author == '@virat.kohli'



# -*- coding: utf-8 -*-
"""
    Instagramy
    ~~~~~~~~~~

    A python package for Instagram. It scarpe the Instagram
    contents.

    :license: MIT License
"""

from .InstagramUser import InstagramUser
from .InstagramPost import InstagramPost
from .InstagramHashTag import InstagramHashTag



""" Html Parsers for various Instagram pages """

import json
from html.parser import HTMLParser

from .exceptions import RedirectionError


class ParseUser(HTMLParser):

    """
    Class Parse the Static Html of the Instagram
    User website and return the required Data as
    Python Dict

    This Class Inherits html.parser.HtmlParser
    """

    Data = ""

    def handle_data(self, data):
        if data.startswith("window._sharedData"):
            try:
                self.Data = json.loads(data[data.find('{"config"') : -1])["entry_data"][
                    "ProfilePage"
                ][0]["graphql"]["user"]
            except (KeyError, json.JSONDecodeError):
                raise RedirectionError
        else:
            pass


class ParsePost(HTMLParser):

    """
    Class Parse the Static Html of the Instagram
    Post website and return the required Data as
    Python Dict

    This Class Inherits html.parser.HtmlParser
    """

    Data = ""

    def handle_data(self, data):
        if '{"@context"' in data:
            try:
                self.Data = json.loads(data.strip())
            except (KeyError, json.JSONDecodeError):
                raise RedirectionError
        else:
            pass


class ParseHashTag(HTMLParser):

    """
    Class Parse the Static Html of the Instagram
    HashTag website and return the required Data as
    Python Dict

    This Class Inherits html.parser.HtmlParser
    """

    Data = ""

    def handle_data(self, data):
        if data.startswith("window._sharedData"):
            try:
                self.Data = json.loads(data[data.find('{"config"') : -1])["entry_data"][
                    "TagPage"
                ][0]["graphql"]["hashtag"]
            except (KeyError, json.JSONDecodeError):
                raise RedirectionError
        else:
            pass



"""
    instagramy.InstagramHashtag
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This module scrape data
    of given Instagram Hashtag.

    Usage Example
    -------------
    ::

        from instagramy.InstagramHashtag import InstagramHashtag

        >>> tag = InstagramHashtag('python')
        >>> tag.number_of_posts
        >>> tag.top_posts

"""

from .core.parser import ParseHashTag
from .core.requests import get
from .core.exceptions import HTTPError, HashTagNotFound


class InstagramHashTag:
    """
    Class InstagramHashTag scrapes instagram hashtag information
    >>> hashtag = InstagramHashTag("python")
    >>> hashtag.number_of_posts
    3119668
    >>> instagram_user.posts_display_urls
    """

    def __init__(self, tag: str):
        self.url = f"https://www.instagram.com/explore/tags/{tag}/"
        self.tag_data = self.get_json()

    def get_json(self) -> dict:
        """
        Return a dict of Hashtag information
        """
        try:
            html = get(self.url)
        except HTTPError:
            raise HashTagNotFound(self.url.split("/")[-2])
        parser = ParseHashTag()
        parser.feed(html)
        return parser.Data

    @property
    def tagname(self) -> str:
        """ Tagname of the Hagtag """
        return self.tag_data["name"]

    @property
    def profile_pic_url(self) -> str:
        """ Profile picture url of the Hagtag """
        return self.tag_data["profile_pic_url"]

    @property
    def number_of_posts(self) -> int:
        """ No.of posts in given Hashtag """
        return self.tag_data["edge_hashtag_to_media"]["count"]

    @property
    def top_posts(self) -> list:
        """
        Top post data (<70) in the given Hashtag
        """

        post_lists = []
        nodes = self.tag_data["edge_hashtag_to_media"]["edges"]
        for node in nodes:
            data = {}
            try:
                data["likes"] = node["node"]["edge_liked_by"]["count"]
            except (KeyError, TypeError):
                data["likes"] = None
            try:
                data["comments"] = node["node"]["edge_media_to_comment"]["count"]
            except (KeyError, TypeError):
                data["comments"] = None
            try:
                data["is_video"] = node["node"]["is_video"]
            except (KeyError, TypeError):
                data["is_video"] = None
            try:
                data["timestamp"] = node["node"]["taken_at_timestamp"]
            except (KeyError, TypeError):
                data["timestamp"] = None
            try:
                data["caption"] = node["node"]["accessibility_caption"]
            except (KeyError, TypeError):
                data["caption"] = None
            try:
                data["shortcode"] = node["node"]["shortcode"]
            except (KeyError, TypeError):
                data["shortcode"] = None
            try:
                data[
                    "post_url"
                ] = f'https://www.instagram.com/p/{node["node"]["shortcode"]}'
            except (KeyError, TypeError):
                data["post_url"] = None
            try:
                data["display_url"] = node["node"]["display_url"]
            except (KeyError, TypeError):
                data["display_url"] = None
            post_lists.append(data)
        return post_lists

    @property
    def posts_display_urls(self) -> list:
        """
        Top post (<70) in the given Hashtag
        """
        return [i["display_url"] for i in self.top_posts]

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}('{self.tagname}')"

    def __str__(self) -> str:
        return f"{'#' + self.tagname} has {self.number_of_posts} posts"



"""
    instagramy.InstagramPost
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This module scrape Instagram Post data
    for given Instagram Post id.

    Usage Example
    -------------
    ::

        from instagramy.InstagramHashtag import InstagramPost

        >>> post = InstagramPost('CGeYX2OA61s')
        >>> post.author
        >>> post.number_of_likes
        >>> post.number_of_comments

"""

from .core.parser import ParsePost
from .core.requests import get
from .core.exceptions import PostIdNotFound, HTTPError


class InstagramPost:
    """
    Class InstagramPost scrape the post information
    by given post id (From url of the post)
    `https://www.instagram.com/p/<post_id>/`
    `https://www.instagram.com/p/CGeYX2OA61s/`

    >>> post = InstagramPost("CGeYX2OA61s")
    >>> post.author
    '@virat.kohli'
    >>> post.number_of_likes
    2203830
    >>> post.number_of_comments
    4629
    """

    def __init__(self, post_id: str):
        self.post_id = post_id
        self.url = f"https://www.instagram.com/p/{post_id}/"
        self.post_details = self.post_detail()

    def post_detail(self) -> dict:
        """
        Return a dict of Post information
        """

        try:
            html = get(self.url)
        except HTTPError:
            raise PostIdNotFound(self.post_id)
        parser = ParsePost()
        parser.feed(html)
        info = parser.Data
        post_details = {}
        try:
            post_details["caption"] = info["caption"]
        except (KeyError, TypeError):
            post_details["caption"] = None
        try:
            post_details["uploaddate"] = info["uploadDate"]
        except (KeyError, TypeError):
            post_details["uploaddate"] = None
        try:
            post_details["author"] = info["author"]["alternateName"]
        except (KeyError, TypeError):
            post_details["author"] = None
        try:
            post_details["profile_page_url"] = info["author"]["mainEntityofPage"]["@id"]
        except (KeyError, TypeError):
            post_details["profile_page_url"] = None
        try:
            post_details["likes"] = info["interactionStatistic"]["userInteractionCount"]
        except (KeyError, TypeError):
            post_details["likes"] = None
        try:
            post_details["comments"] = info["commentCount"]
        except (KeyError, TypeError):
            post_details["comments"] = None
        try:
            post_details["description"] = info["description"]
        except (KeyError, TypeError):
            post_details["description"] = None

        return post_details

    @property
    def number_of_likes(self) -> int:
        """ No.of Like is given post """
        return int(self.post_details["likes"])

    @property
    def number_of_comments(self) -> int:
        """ No.of Comments is given post """
        return int(self.post_details["comments"])

    @property
    def author(self) -> str:
        """ Author of the Post """
        return self.post_details["author"]

    @property
    def caption(self) -> str:
        """ Caption of the Post """
        return self.post_details["caption"]

    @property
    def description(self) -> str:
        """ Discription of the Post given by Instagram """
        return self.post_details["description"]

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}('{self.post_id}')"

    def __str__(self) -> str:
        return f"Post ({self.post_id}) posted by {self.author} with {self.number_of_likes} likes"



""" Wrapper for urllib.request """

from urllib.request import Request
from urllib.request import urlopen

from .user_agent import headers


def get(url: str) -> str:
    """
    Function send the HTTP requests to given site
    and return the html content of the webpage
    """

    request = Request(url, headers=headers)

    with urlopen(request) as reponse:
        html = reponse.read()

    return html.decode("utf-8")
